@@ -33,6 +33,7 @@ MOCK_MODULES = [
     "tqdm",
     "simpleppt",
     "sklearn.metrics",
+    "networkx",
 ]
 for mod_name in MOCK_MODULES:
     sys.modules[mod_name] = mock.Mock()
@@ -126,7 +127,6 @@ notebooks = [
     "Advanced_bifurcation_analysis.ipynb",
     "Tree_operations.ipynb",
     "Conversion_from_CellRank_pipeline.ipynb",
-    "Critical_Transition.ipynb",
     "Beyond_scRNAseq.ipynb",
 ]
 for nb in notebooks:
