@@ -5,19 +5,27 @@ from anndata import AnnData
 import igraph
 import seaborn as sns
 import matplotlib.pyplot as plt
-from matplotlib.colors import to_hex
 from scipy import stats
 from adjustText import adjust_text
 from matplotlib import patches
 from scipy import sparse
+import matplotlib.gridspec as gridspec
 
 from typing import Union, Optional, List
 from typing_extensions import Literal
 from scanpy.plotting._utils import savefig_or_show
 
 from .. import logging as logg
-from ..tools.utils import getpath
+from ..tools.utils import getpath, importeR
 from .trajectory import trajectory
+from .dendrogram import dendrogram
+
+Rpy2, R, rstats, rmgcv, Formula = importeR("fitting associated features")
+check = [type(imp) == str for imp in [Rpy2, R, rstats, rmgcv, Formula]]
+
+from .modules import get_modules
+from .utils import gen_milestones_gradients
+from .. import logging as logg
 
 
 def trends(
@@ -29,7 +37,7 @@ def trends(
     milestones: Union[None, str] = None,
     module: Union[None, Literal["early", "late"]] = None,
     branch: Union[None, str] = None,
-    annot: Union[None, Literal["seg", "milestones"]] = None,
+    annot: Union[None, str] = None,
     title: str = "",
     feature_cmap: str = "RdBu_r",
     pseudo_cmap: str = "viridis",
@@ -37,7 +45,8 @@ def trends(
     basis: str = "umap",
     heatmap_space: float = 0.5,
     offset_names: float = 0.15,
-    fontsize: float = 9,
+    fontsize: int = 9,
+    style: Literal["normal", "italic", "oblique"] = "normal",
     ordering: Union[
         None, Literal["pearson", "spearman", "quantile", "max"]
     ] = "pearson",
@@ -47,6 +56,7 @@ def trends(
     complex_z: float = 3,
     fig_heigth: float = 4,
     show: Optional[bool] = None,
+    output_mean: bool = False,
     save: Union[str, bool, None] = None,
     save_genes: Optional[bool] = None,
     **kwargs,
@@ -74,7 +84,7 @@ def trends(
     branch
         if bifurcation analysis as been performed, subset features to a specific milestone.
     annot
-        adds an annotation row o ntop of the heatmap.
+        adds an annotation row on top of the heatmap.
     title
         add a title.
     feature_cmap
@@ -91,6 +101,8 @@ def trends(
         how far on the right the annotated features should be displayed, in proportion of the heatmap space.
     fontsize
         font size of the feature annotations.
+    style
+        font style.
     ordering
         strategy to order the features on heatmap, quantile takes the mean pseudotime of the choosen value.
     ord_thre
@@ -105,6 +117,8 @@ def trends(
         standard deviation threshold on the quantile subsetted feature.
     fig_heigth
         figure height.
+    output_mean
+        output mean fitted values to adata.
     show
         show the plot.
     save
@@ -112,7 +126,7 @@ def trends(
     save_genes
         save list of genes following the order displayed on the heatmap.
     **kwargs
-        arguments passed to :func:`scFates.pl.trajectory` if `plot_emb=True`
+        if `plot_emb=True`, arguments passed to :func:`scFates.pl.trajectory` or :func:`scFates.pl.dendrogram` if `basis="dendro"`
 
     Returns
     -------
@@ -268,79 +282,45 @@ def trends(
     fitted_sorted = fitted.loc[feature_order, :]
 
     if annot == "milestones":
-        color_key = "milestones_colors"
-        if color_key not in adata.uns or len(adata.uns[color_key]) == 1:
-            from . import palette_tools
+        annot_cmap = gen_milestones_gradients(adata, seg_order)
 
-            palette_tools._set_default_colors_for_categorical_obs(adata, "milestones")
+    elif type(annot) == str:
+        if len(adata.obs[annot].unique()) > 1:
+            color_key = annot + "_colors"
+            if color_key not in adata.uns or len(adata.uns[color_key]) == 1:
+                from . import palette_tools
 
-        def milestones_prog(s):
-            cfrom = adata.obs.t[adata.obs.seg == s].idxmin()
-            cto = adata.obs.t[adata.obs.seg == s].idxmax()
-            mfrom = adata.obs.milestones[cfrom]
-            mto = adata.obs.milestones[cto]
-            import numpy as np
-
-            mfrom_c = adata.uns["milestones_colors"][
-                np.argwhere(adata.obs.milestones.cat.categories == mfrom)[0][0]
-            ]
-            mto_c = adata.uns["milestones_colors"][
-                np.argwhere(adata.obs.milestones.cat.categories == mto)[0][0]
-            ]
-
-            from matplotlib.colors import LinearSegmentedColormap
-
-            cm = LinearSegmentedColormap.from_list("test", [mfrom_c, mto_c], N=1000)
-            pst = (
-                adata.obs.t[adata.obs.seg == s] - adata.obs.t[adata.obs.seg == s].min()
-            ) / (
-                adata.obs.t[adata.obs.seg == s].max()
-                - adata.obs.t[adata.obs.seg == s].min()
+                palette_tools._set_default_colors_for_categorical_obs(adata, annot)
+            annot_cmap = pd.Series(
+                list(
+                    map(
+                        lambda c: adata.uns[color_key][
+                            adata.obs[annot].cat.categories == c
+                        ][0],
+                        adata.obs[annot][fitted_sorted.columns].values,
+                    )
+                ),
+                index=fitted_sorted.columns,
             )
-            return pd.Series(list(map(to_hex, cm(pst))), index=pst.index)
+    else:
+        annot = None
 
-        annot_cmap = pd.concat(list(map(milestones_prog, seg_order)))
+    ratio = plt.rcParams["figure.figsize"][0] / plt.rcParams["figure.figsize"][1]
+    fig = plt.figure(figsize=(fig_heigth + (fig_heigth * plot_emb * ratio), fig_heigth))
+    outer = gridspec.GridSpec(1, 2, figure=fig, width_ratios=[1 * plot_emb * ratio, 1])
 
-    if annot == "seg":
-        color_key = "seg_colors"
-        if color_key not in adata.uns or len(adata.uns[color_key]) == 1:
-            from . import palette_tools
-
-            palette_tools._set_default_colors_for_categorical_obs(adata, "seg")
-
-        annot_cmap = pd.Series(
-            list(
-                map(
-                    lambda s: adata.uns["seg_colors"][
-                        adata.obs.seg.cat.categories == s
-                    ][0],
-                    adata.obs.seg[fitted_sorted.columns].values,
-                )
-            ),
-            index=fitted_sorted.columns,
-        )
-
-    fig, f_axs = plt.subplots(
-        ncols=2,
-        nrows=20,
-        figsize=(fig_heigth * (1 + 1 * plot_emb), fig_heigth),
-        gridspec_kw={"width_ratios": [1 * plot_emb, 1]},
+    gs_ht = gridspec.GridSpecFromSubplotSpec(
+        2 + (annot is not None),
+        1,
+        height_ratios=(1, 1, 18) if (annot is not None) else (1, 19),
+        subplot_spec=outer[1],
+        hspace=0,
     )
-    gs = f_axs[2, 1].get_gridspec()
-
-    # remove the underlying axes
-    start = 2 if annot is not None else 1
-    for ax in f_axs[start:, -1]:
-        ax.remove()
-    axheatmap = fig.add_subplot(gs[start:, -1])
-
-    gs = f_axs[0, 0].get_gridspec()
-    # remove the underlying axes
-    for ax in f_axs[:, 0]:
-        ax.remove()
-
+    axs = []
+    i = 0
     if annot is not None:
-        axannot = f_axs[0, 1]
+        axannot = plt.subplot(gs_ht[i])
+        axs = axs + [axannot]
         sns.heatmap(
             pd.DataFrame(range(fitted_sorted.shape[1])).T,
             robust=False,
@@ -351,11 +331,10 @@ def trends(
             cbar=False,
             ax=axannot,
         )
-        axpsdt = f_axs[1, 1]
-
-    else:
-        axpsdt = f_axs[0, 1]
+        i = i + 1
 
+    axpsdt = plt.subplot(gs_ht[i])
+    axs = axs + [axpsdt]
     sns.heatmap(
         pd.DataFrame(adata.obs.t[fitted_sorted.columns].values).T,
         robust=True,
@@ -368,6 +347,8 @@ def trends(
         vmax=adata.obs.t.max(),
     )
 
+    axheatmap = plt.subplot(gs_ht[i + 1])
+    axs = axs + [axheatmap]
     sns.heatmap(
         fitted_sorted,
         robust=True,
@@ -435,7 +416,7 @@ def trends(
 
     texts = []
     for x, y, s in zip(xs, ys, highlight_features):
-        texts.append(axheatmap.text(x, y, s, fontsize=fontsize))
+        texts.append(axheatmap.text(x, y, s, fontsize=fontsize, style=style))
 
     patch = patches.Rectangle(
         (0, 0),
@@ -477,26 +458,51 @@ def trends(
         yy = [ys[i], texts[i].get_position()[1]]
         axheatmap.plot(xx, yy, color="k", linewidth=0.75)
 
-    plt.tight_layout(h_pad=0, w_pad=0)
-
     if plot_emb:
-        axemb = fig.add_subplot(gs[:, 0])
-        adata_temp.obs["mean_trajectory"] = np.nan
+        gs_emb = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=outer[0])
+        axemb = fig.add_subplot(gs_emb[0])
+        axs = axs + [axemb]
+        adata_temp.obs["mean_trajectory"] = 0
         adata_temp.obs.loc[
             fitted_sorted.columns, "mean_trajectory"
         ] = fitted_sorted.mean(axis=0).values
-        trajectory(
-            adata=adata_temp,
-            basis=basis,
-            color_seg="mean_trajectory",
-            cmap_seg=feature_cmap,
-            show_info=False,
-            ax=axemb,
-            title=title,
-            root_milestone=root_milestone,
-            milestones=milestones,
-            **kwargs,
-        )
+        if output_mean:
+            adata.obs["mean_trajectory"] = adata_temp.obs["mean_trajectory"]
+        if "color" in kwargs:
+            color = kwargs["color"]
+            del kwargs["color"]
+        else:
+            color = "mean_trajectory"
+
+        if "cmap" in kwargs:
+            feature_cmap = kwargs["cmap"]
+            del kwargs["cmap"]
+
+        if basis == "dendro":
+            dendrogram(
+                adata_temp,
+                color=color,
+                cmap=feature_cmap,
+                ax=axemb,
+                title=title,
+                root_milestone=root_milestone,
+                milestones=milestones,
+                show_info=False,
+                **kwargs,
+            )
+        else:
+            trajectory(
+                adata=adata_temp,
+                basis=basis,
+                color_seg=color,
+                cmap_seg=feature_cmap,
+                show_info=False,
+                ax=axemb,
+                title=title,
+                root_milestone=root_milestone,
+                milestones=milestones,
+                **kwargs,
+            )
 
     if save_genes is not None:
         with open(save_genes, "w") as f:
@@ -504,14 +510,20 @@ def trends(
                 f.write("%s\n" % item)
 
     if show == False:
-        return f_axs
-
-    savefig_or_show("trends", show=show, save=save)
+        return axs
+    if save is not None:
+        savefile = "figures/trends" + save
+        logg.warn("saving figure to file " + savefile)
+        fig.savefig(savefile, bbox_inches="tight")
 
 
 def single_trend(
     adata: AnnData,
-    feature: str,
+    feature: Union[str, None] = None,
+    root_milestone: Union[None, str] = None,
+    milestones: Union[None, str] = None,
+    module: Union[None, Literal["early", "late"]] = None,
+    branch: Union[None, str] = None,
     basis: str = "umap",
     ylab: str = "expression",
     color_exp=None,
@@ -521,6 +533,7 @@ def single_trend(
     layer: Union[str, None] = None,
     cmap_seg: str = "RdBu_r",
     cmap_cells: str = "RdBu_r",
+    plot_emb: bool = True,
     figsize: tuple = (8, 4),
     show: Optional[bool] = None,
     save: Union[str, bool, None] = None,
@@ -535,7 +548,15 @@ def single_trend(
     adata
         Annotated data matrix.
     feature
-        Name of the fitted feature
+        Name of the fitted feature.
+    root_milestone
+        if plotting module instead of feature, tip defining progenitor branch.
+    milestones
+        if plotting module instead of feature, tips defining the progenies branches.
+    module
+        if plotting module instead of feature, whether to plot early or late modules.
+    branch
+        if plotting module instead of feature, plot fitted milestone-specific module.
     basis
         Name of the `obsm` basis to use.
     ylab
@@ -567,37 +588,74 @@ def single_trend(
 
     """
 
-    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=figsize, constrained_layout=True)
+    if root_milestone is None:
+        color_key = "seg_colors"
+        if color_key not in adata.uns:
+            from . import palette_tools
 
-    color_key = "seg_colors"
-    if color_key not in adata.uns:
-        from . import palette_tools
+            palette_tools._set_default_colors_for_categorical_obs(adata, "seg")
 
-        palette_tools._set_default_colors_for_categorical_obs(adata, "seg")
-
-    if layer is None:
-        if sparse.issparse(adata.X):
-            Xfeature = adata[:, feature].X.A.T.flatten()
+        if layer is None:
+            if sparse.issparse(adata.X):
+                Xfeature = adata[:, feature].X.A.T.flatten()
+            else:
+                Xfeature = adata[:, feature].X.T.flatten()
         else:
-            Xfeature = adata[:, feature].X.T.flatten()
+            if sparse.issparse(adata.layers[layer]):
+                Xfeature = adata[:, feature].layers[layer].A.T.flatten()
+            else:
+                Xfeature = adata[:, feature].layers[layer].T.flatten()
+
+        df = pd.DataFrame(
+            {
+                "t": adata.obs.t,
+                "fitted": adata[:, feature].layers["fitted"].flatten(),
+                "expression": Xfeature,
+                "seg": adata.obs.seg,
+            }
+        ).sort_values("t")
     else:
-        if sparse.issparse(adata.layers[layer]):
-            Xfeature = adata[:, feature].layers[layer].A.T.flatten()
-        else:
-            Xfeature = adata[:, feature].layers[layer].T.flatten()
+        if plot_emb:
+            logg.warn("trajectory plotting is not yet implemented for module plot!")
+            plot_emb = False
+        graph = adata.uns["graph"]
+        miles_cat = adata.obs.milestones.cat.categories
+        mil_col = np.array(adata.uns["milestones_colors"])
 
-    df = pd.DataFrame(
-        {
-            "t": adata.obs.t,
-            "fitted": adata[:, feature].layers["fitted"].flatten(),
-            "expression": Xfeature,
-            "seg": adata.obs.seg,
-        }
-    ).sort_values("t")
+        X_early, X_late = get_modules(adata, root_milestone, milestones, layer)
+        X = pd.concat([X_early, X_late], axis=1)
+        X["t"] = adata.obs.loc[X.index, "t"].values
+
+        def gamfit(f):
+            m = rmgcv.gam(
+                Formula(f),
+                data=X,
+                gamma=1.5,
+            )
+            return pd.Series(rmgcv.predict_gam(m), index=X.index)
+
+        predictions = gamfit("{}_{}~s(t,bs='ts')".format(module, branch))
+
+        df = pd.DataFrame(
+            {
+                "t": adata[X.index].obs.t,
+                "fitted": predictions.values,
+                "expression": X["_".join([module, branch])].values,
+                "seg": adata[X.index].obs.seg,
+            }
+        ).sort_values("t")
+
+        color_exp = mil_col[miles_cat == branch][0]
+
+    if plot_emb:
+        fig, axs = plt.subplots(1, 2, figsize=figsize)
+    else:
+        fig, axs = plt.subplots(1, 1, figsize=figsize)
+        axs = ["empty", axs]
 
     for s in df.seg.unique():
         if color_exp is None:
-            ax2.scatter(
+            axs[1].scatter(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "expression"],
                 alpha=alpha_expr,
@@ -606,7 +664,7 @@ def single_trend(
                     np.argwhere(adata.obs.seg.cat.categories == s)[0][0]
                 ],
             )
-            ax2.plot(
+            axs[1].plot(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "fitted"],
                 c=adata.uns["seg_colors"][
@@ -620,7 +678,7 @@ def single_trend(
                     adata.uns["graph"]["pp_seg"].loc[:, "from"].isin([tolink]).values
                 ).flatten()
             ]:
-                ax2.plot(
+                axs[1].plot(
                     [
                         df.loc[df.seg == s, "t"].iloc[-1],
                         df.loc[df.seg == next_s, "t"].iloc[0],
@@ -635,14 +693,14 @@ def single_trend(
                     linewidth=fitted_linewidth,
                 )
         else:
-            ax2.scatter(
+            axs[1].scatter(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "expression"],
                 c=color_exp,
                 alpha=alpha_expr,
                 s=size_expr,
             )
-            ax2.plot(
+            axs[1].plot(
                 df.loc[df.seg == s, "t"],
                 df.loc[df.seg == s, "fitted"],
                 c=color_exp,
@@ -654,7 +712,7 @@ def single_trend(
                     adata.uns["graph"]["pp_seg"].loc[:, "from"].isin([tolink]).values
                 ).flatten()
             ]:
-                ax2.plot(
+                axs[1].plot(
                     [
                         df.loc[df.seg == s, "t"].iloc[-1],
                         df.loc[df.seg == next_s, "t"].iloc[0],
@@ -667,29 +725,31 @@ def single_trend(
                     linewidth=fitted_linewidth,
                 )
 
-    ax2.set_ylabel(ylab)
-    ax2.set_xlabel("pseudotime")
-    x0, x1 = ax2.get_xlim()
-    y0, y1 = ax2.get_ylim()
-    ax2.set_aspect(abs(x1 - x0) / abs(y1 - y0))
+    axs[1].set_ylabel(ylab)
+    axs[1].set_xlabel("pseudotime")
+    x0, x1 = axs[1].get_xlim()
+    y0, y1 = axs[1].get_ylim()
+    if plot_emb:
+        axs[1].set_aspect(abs(x1 - x0) / abs(y1 - y0))
 
-    trajectory(
-        adata,
-        basis=basis,
-        color_seg=feature,
-        cmap_seg=cmap_seg,
-        color_cells=feature,
-        cmap=cmap_cells,
-        show_info=False,
-        ax=ax1,
-        title=feature,
-        layer="fitted",
-        show=False,
-        save=False,
-        **kwargs,
-    )
+    if plot_emb:
+        trajectory(
+            adata,
+            basis=basis,
+            color_seg=feature,
+            cmap_seg=cmap_seg,
+            color_cells=feature,
+            cmap=cmap_cells,
+            show_info=False,
+            ax=axs[0],
+            title=feature,
+            layer="fitted",
+            show=False,
+            save=False,
+            **kwargs,
+        )
 
     savefig_or_show("single_trend", show=show, save=save)
 
     if show is False:
-        return (ax1, ax2)
+        return axs if plot_emb else axs[1]
