@@ -24,6 +24,10 @@ import sys
 
 sys.setrecursionlimit(10000)
 
+from scFates.tools.utils import importeR
+
+Rpy2, R, rstats, rmgcv, Formula = importeR("Fitting syncho path on cells")
+
 
 def synchro_path(
     adata: AnnData,
@@ -38,7 +42,7 @@ def synchro_path(
     w=200,
     step=30,
     winp=10,
-    loess_span=0.2,
+    knots=10,
     copy: bool = False,
 ):
     """\
@@ -66,8 +70,8 @@ def synchro_path(
         steps, in number of cells, between local windows.
     winp
         window of permutation in cells.
-    loess_span
-        fraction of points to take in account for loess fit
+    knots
+        number of knots for GAM fit of corAB on cells pre-fork
     copy
         Return a copy instead of writing to adata.
 
@@ -227,22 +231,6 @@ def synchro_path(
             + [milestones[0] + " vs " + milestones[1] + "\ninter-module"],
         )
     )
-    logg.info(" done, computing LOESS fit")
-    for cc in ["corAA", "corBB", "corAB"]:
-        allcor[cc + "_lowess"] = 0
-        allcor[cc + "_ll"] = 0
-        allcor[cc + "_ul"] = 0
-        for r in range(len(runs)):
-            for mil in milestones:
-                res = allcor.loc[runs[r]].loc[mil]
-                l = loess(res.t, res[cc], span=loess_span)
-                l.fit()
-                pred = l.predict(res.t, stderror=True)
-                conf = pred.confidence()
-
-                allcor.loc[(runs[r], mil), cc + "_lowess"] = pred.values
-                allcor.loc[(runs[r], mil), cc + "_ll"] = conf.lower
-                allcor.loc[(runs[r], mil), cc + "_ul"] = conf.upper
 
     fork = list(
         set(img.get_shortest_paths(str(root), str(leaves[0]))[0]).intersection(
@@ -253,16 +241,19 @@ def synchro_path(
     fork_t = adata.uns["graph"]["pp_info"].loc[fork, "time"].max()
     res = allcor.loc[allcor.t < fork_t, :]
     res = res[~res.t.duplicated()]
-    l = loess(res.t, res["corAB"], span=loess_span)
-    l.fit()
-    pred = l.predict(res.t, stderror=True)
+
+    m = rmgcv.gam(
+        Formula("corAB ~ s(t, bs = 'cs',k=%s)" % knots),
+        data=res,
+    )
+    pred = rmgcv.predict_gam(m)
 
     tval = adata.obs.t.copy()
     tval[tval > fork_t] = np.nan
 
     def inter_values(tv):
         if ~np.isnan(tv):
-            return pred.values[np.argmin(np.abs(res.t.values - tv))]
+            return pred[np.argmin(np.abs(res.t.values - tv))]
         else:
             return tv
 
@@ -311,7 +302,7 @@ def synchro_path(
         + "']['synchro'], mean local gene-gene correlations of all possible gene pairs inside one module, or between the two modules.\n"
         "    .obs['inter_cor "
         + name
-        + "'], loess fit of inter-module mean local gene-gene correlations prior to bifurcation."
+        + "'], GAM fit of inter-module mean local gene-gene correlations prior to bifurcation."
     )
 
     return adata if copy else None
@@ -757,366 +748,3 @@ def module_inclusion(
     )
 
     return adata if copy else None
-
-
-def critical_transition(
-    adata: AnnData,
-    root_milestone,
-    milestones,
-    n_map=1,
-    n_jobs=None,
-    layer: Optional[str] = None,
-    w=100,
-    step=30,
-    loess_span=0.4,
-    gamma=1.5,
-    n_points=200,
-    copy: bool = False,
-):
-    """\
-    Estimates local critical transition index along the trajectory.
-
-    Based from the concept of pre-bifurcation struture from [Bargaje17]_.
-    This study proposes the idea that a signature indicating the flattening
-    of the quasi-potential landscape can be detected prior to bifurcation.
-
-    To detect this signal, this function estimates local critical transition
-    index along the trajectory, by calculating along a moving window of cell
-    the following:
-
-    .. math::
-        \\frac{<{\\left | R(g_i,g_j) \\right |>}}{<\\left | R(c_k,c_l) \\right |>}
-
-    Which is the ratio between the mean of the absolute gene by gene correlations
-    and the mean of the absolute cell by cell correlations.
-
-    Parameters
-    ----------
-    adata
-        Annotated data matrix.
-    root_milestone
-        tip defining progenitor branch.
-    milestones
-        tips defining the progenies branches.
-    n_map
-        number of probabilistic cells projection to use for estimates.
-    n_jobs
-        number of cpu processes to perform estimates (per mapping).
-    layer
-        adata layer to use for estimates.
-    w
-        local window, in number of cells, to estimate correlations.
-    step
-        steps, in number of cells, between local windows.
-    loess_span
-        fraction of points to take in account for loess fit
-    copy
-        Return a copy instead of writing to adata.
-
-    Returns
-    -------
-    adata : anndata.AnnData
-        if `copy=True` it returns subsetted or else subset (keeping only
-        significant features) and add fields to `adata` for a bifurcation:
-
-        `.uns['root_milestone->milestoneA<>milestoneB']['critical transition']`
-            containing local critical transition index per window of cells.
-        `.obs['root_milestone->milestoneA<>milestoneB pre-fork CI lowess']`
-            local critical transition index loess fitted onto cells prior to bifurcation.
-
-    For a linear trajectory:
-
-        `.uns['root_milestone->milestoneA']['critical transition']`
-            containing local critical transition index per window of cells.
-        `.obs['root_milestone->milestoneA CI lowess']`
-            local critical transition index loess fitted onto cells along the path.
-
-    """
-
-    adata = adata.copy() if copy else adata
-
-    logg.info("Calculating local critical transition index", reset=True)
-
-    graph = adata.uns["graph"]
-
-    edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
-    img = igraph.Graph()
-    img.add_vertices(
-        np.unique(graph["pp_seg"][["from", "to"]].values.flatten().astype(str))
-    )
-    img.add_edges(edges)
-
-    uns_temp = adata.uns.copy()
-
-    if "milestones_colors" in adata.uns:
-        mlsc = adata.uns["milestones_colors"].copy()
-
-    dct = graph["milestones"]
-    keys = np.array(list(dct.keys()))
-    vals = np.array(list(dct.values()))
-
-    leaves = list(map(lambda leave: dct[leave], milestones))
-    root = dct[root_milestone]
-
-    name = root_milestone + "->" + "<>".join(milestones)
-
-    def critical_map(m, gamma, loess_span):
-        df = adata.uns["pseudotime_list"][str(m)]
-        edges = graph["pp_seg"][["from", "to"]].astype(str).apply(tuple, axis=1).values
-        img = igraph.Graph()
-        img.add_vertices(
-            np.unique(graph["pp_seg"][["from", "to"]].values.flatten().astype(str))
-        )
-        img.add_edges(edges)
-
-        def critical_milestone(leave):
-            cells = getpath(img, root, graph["tips"], leave, graph, df).index
-
-            X = get_X(adata, cells, adata.var_names, layer)
-            mat = pd.DataFrame(X, index=cells, columns=adata.var_names)
-
-            mat = mat.iloc[adata.obs.t[mat.index].argsort().values, :]
-
-            def slide_path(i):
-                cls = mat.index[i : (i + w)]
-                cor_gene = mat.loc[cls, :].corr(method="pearson").values
-                cor_cell = mat.loc[cls, :].T.corr(method="pearson").values
-                R_gene = np.nanmean(
-                    np.abs(cor_gene[np.triu_indices(cor_gene.shape[0], k=1)])
-                )
-                R_cell = np.nanmean(
-                    np.abs(cor_cell[np.triu_indices(cor_cell.shape[0], k=1)])
-                )
-                return [adata.obs.t[cls].mean(), R_gene / R_cell, cls]
-
-            wins = np.arange(0, mat.shape[0] - w, step)
-
-            stats = ProgressParallel(
-                n_jobs=n_jobs,
-                total=len(wins),
-                use_tqdm=n_map == 1,
-                file=sys.stdout,
-                desc="    to " + str(keys[vals == leave][0]),
-            )(delayed(slide_path)(i) for i in wins)
-
-            cells_l = [s[2] for s in stats]
-            stats = pd.DataFrame([[s[0], s[1]] for s in stats], columns=("t", "ci"))
-
-            l = loess(stats.t, stats.ci, span=loess_span)
-            l.fit()
-            pred = l.predict(stats.t, stderror=True)
-            conf = pred.confidence()
-
-            stats["lowess"] = pred.values
-            stats["ll"] = conf.lower
-            stats["ul"] = conf.upper
-
-            cell_stats = [
-                pd.DataFrame(
-                    np.repeat(stats.ci[i].reshape(-1, 1), len(cells_l[i])),
-                    index=cells_l[i],
-                    columns=["ci"],
-                )
-                for i in range(stats.shape[0])
-            ]
-
-            cell_stats = pd.concat(cell_stats, axis=1)
-            cell_stats = cell_stats.T.groupby(level=0).mean().T
-            cell_stats["t"] = adata.obs.loc[cell_stats.index, "t"]
-
-            l = loess(cell_stats.t, cell_stats.ci, span=loess_span)
-            pred = l.predict(cell_stats.t, stderror=True)
-
-            cell_stats["fit"] = pred.values
-
-            lspaced_stats = pd.DataFrame(
-                {
-                    "t": np.linspace(
-                        cell_stats["t"].min(), cell_stats["t"].max(), n_points
-                    )
-                }
-            )
-            pred = l.predict(lspaced_stats.t, stderror=True)
-            lspaced_stats["fit"] = pred.values
-
-            del cell_stats["t"]
-            return stats, cell_stats, lspaced_stats
-
-        res = list(map(critical_milestone, leaves))
-
-        cell_stats = pd.concat([r[1] for r in res]).groupby(level=0).mean()
-
-        res_slide = dict(zip(milestones, [r[0] for r in res]))
-
-        res_lspaced = dict(zip(milestones, [r[2] for r in res]))
-
-        return cell_stats, res_slide, res_lspaced
-
-    if n_map == 1:
-        df, res_slide, res_lspaced = critical_map(0, gamma, loess_span)
-    else:
-        # TODO: adapt multimapping
-        stats = Parallel(n_jobs=n_jobs)(
-            delayed(critical_map)(i)
-            for i in tqdm(range(n_map), file=sys.stdout, desc="    multi mapping ")
-        )
-        res_slides = pd.concat(stats)
-
-    if name in adata.uns:
-        adata.uns[name]["critical transition"] = {
-            "LOESS": res_slide,
-            "eLOESS": res_lspaced,
-        }
-    else:
-        adata.uns[name] = {
-            "critical transition": {"LOESS": res_slide, "eLOESS": res_lspaced}
-        }
-
-    adata.obs.loc[df.index, name + " CI"] = df.ci.values
-
-    adata.obs.loc[df.index, name + " CI fitted"] = df.fit.values
-
-    logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
-    logg.hint(
-        "added \n"
-        "    .uns['"
-        + name
-        + "']['critical transition'], df containing local critical transition index per window of cells.\n"
-        "    .obs['"
-        + name
-        + " CI'], local critical transition index projected onto cells.\n"
-        "    .obs['"
-        + name
-        + " CI fitted'], GAM fit of local critical transition index projected onto cells."
-    )
-
-    return adata if copy else None
-
-
-def criticality_drivers(
-    adata: AnnData,
-    root_milestone,
-    milestones,
-    t_span=None,
-    confidence_level: float = 0.95,
-    layer: Optional[str] = None,
-    device="cpu",
-    copy: bool = False,
-):
-
-    """\
-    Calculates correlations between genes and local critical transition index along trajectory.
-
-    Fisher test for the correlations comes from CellRank function `cr.tl.lineages_drivers`.
-
-    Parameters
-    ----------
-    adata
-        Annotated data matrix.
-    root_milestone
-        tip defining progenitor branch.
-    milestones
-        tips defining the progenies branches.
-    t_span
-        restrict correlations to a window of pseudotime
-    confidence_level
-        correlation confidence interval.
-    layer
-        adata layer to use for estimates.
-    device
-        whether to run the correlation matrix computation on a cpu or gpu.
-    loess_span
-        fraction of points to take in account for loess fit
-    copy
-        Return a copy instead of writing to adata.
-
-    Returns
-    -------
-    adata : anndata.AnnData
-        if `copy=True` it returns subsetted or else subset (keeping only
-        significant features) and add fields to `adata`:
-
-        `.uns['root_milestone->milestoneA<>milestoneB']['criticality drivers'].
-            a df containing gene correlation with critical index transition.
-
-
-    """
-
-    adata = adata.copy() if copy else adata
-
-    logg.info("Calculating gene to critical transition index correlations", reset=True)
-
-    name = root_milestone + "->" + "<>".join(milestones)
-    obs_name = name + " CI fitted"
-
-    if t_span is None:
-        cells = adata.obs_names[~np.isnan(adata.obs[obs_name])]
-    else:
-        cells = adata.obs_names[
-            (~np.isnan(adata.obs[obs_name]))
-            & (adata.obs.t > t_span[0])
-            & (adata.obs.t < t_span[1])
-        ]
-
-    CI = adata[cells].obs[obs_name].values
-
-    if layer is None:
-        X = adata[cells].X
-    else:
-        X = adata[cells].layers[layer]
-
-    if device == "cpu":
-        from .utils import cor_mat_cpu
-
-        X = X.A if sparse.issparse(X) else X
-        corr = cor_mat_cpu(X, CI.reshape(-1, 1)).ravel()
-    else:
-        from .utils import cor_mat_gpu
-        import cupy as cp
-        from cupyx.scipy.sparse import csr_matrix as csr_gpu
-
-        X = csr_gpu(X) if sparse.issparse(X) else cp.array(X)
-        corr = cor_mat_gpu(X, cp.array(CI).reshape(-1, 1)).ravel().get()
-
-    ### Fisher testing of correlations, CellRank implementation
-    ### https://github.com/theislab/cellrank/blob/b6345d5e6dd148317782ffc9a9f96793ad98ead9/cellrank/tl/_utils.py#L488
-    ### Copyright (c) 2019, Theis Lab
-
-    confidence_level = 0.95
-    n = adata.shape[0]
-    ql = 1 - confidence_level - (1 - confidence_level) / 2.0
-    qh = confidence_level + (1 - confidence_level) / 2.0
-
-    mean, se = np.arctanh(corr), 1.0 / np.sqrt(n - 3)
-    z_score = (np.arctanh(corr) - np.arctanh(0)) * np.sqrt(n - 3)
-
-    z = norm.ppf(qh)
-    corr_ci_low = np.tanh(mean - z * se)
-    corr_ci_high = np.tanh(mean + z * se)
-    pvals = 2 * norm.cdf(-np.abs(z_score))
-
-    ###
-
-    res = pd.DataFrame(
-        {"corr": corr, "pval": pvals, "ci_low": corr_ci_low, "ci_high": corr_ci_high},
-        index=adata.var_names,
-    )
-
-    res["q_val"] = np.nan
-    res.loc[~np.isnan(pvals), "q_val"] = multipletests(
-        res[~np.isnan(pvals)].pval.values, alpha=0.05, method="fdr_bh"
-    )[1]
-
-    adata.uns[name]["criticality drivers"] = res.sort_values(
-        "corr", ascending=False
-    ).dropna()
-
-    logg.info("    finished", time=True, end=" " if settings.verbosity > 2 else "\n")
-    logg.hint(
-        "added \n"
-        "    .uns['"
-        + name
-        + "']['criticality drivers'], df containing gene correlation with critical index transition."
-    )
-
-    return adata if copy else None
