@@ -4,7 +4,7 @@ os.environ["OPENBLAS_NUM_THREADS"] = "1"
 os.environ["OMP_NUM_THREADS"] = "1"
 os.environ["MKL_NUM_THREADS"] = "1"
 
-from typing import Union, Optional, Tuple, Collection, Sequence, Iterable
+from typing import Union, Optional, Iterable
 
 import numpy as np
 import pandas as pd
